<?php

declare(strict_types=1);

namespace Cin\Events5;

use Exception;
use SimpleXMLElement;
use stdClass;
use Throwable;

date_default_timezone_set('Europe/Berlin');

final class Licence
{
  private string $licenceCenterURL;

  private string $hosting;

  private string $licenceFileName;

  private string $pluginid;

  private string $version;

  private string $deviceIdentifier;

  private string $publicKey = "-----BEGIN PUBLIC KEY-----\n"
  ."MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAthtgP5X/VKMED+XPva3W\n"
  ."55uek1OtYvvzyJht+i3qN7e3yKJ1nKdxXzrx7MyfOVsGMyjk4MkZIyvXftv74iry\n"
  ."nJ4Kd6tFPGvyk3QT8N3dJu8pM3y4BFtXoUe3Q6pBqjCOHdwzm4c9L5G8AhW6ACGw\n"
  ."Iu/nR/4hxsJPuMF9QgHtPXphYtLZX4e5ZX5wQ65ZQ1Xsl9wENx8Woe8VK/Zgk2HX\n"
  ."UHWHyJvcQ5U2tkxG6kEOQ7kA5opv1u5VwE2gbGIbFMU/tBuhjvt/dADpIMZTXmNc\n"
  ."1VPMrDdMS0eA8AG1BWg85awcKBGgyWWJd2DGMo1Jbjyi0LtxEkKaU7iQ8HK6/uUX\n"
  ."twIDAQAB\n"
  .'-----END PUBLIC KEY-----';

  private object $licence;

  private ?string $exsID = null;

  public function __construct()
  {
    if ( ! \defined('URL_SHOP')) {
      require \dirname(__DIR__, 3).'/includes/config.JTL-Shop.ini.php';
    }

    $infoxml                = new SimpleXMLElement(file_get_contents(\dirname(__DIR__, 1).'/info.xml'));
    $this->pluginid         = (string) $infoxml->PluginID[0];
    $this->version          = (string) $infoxml->Version;
    $this->hosting          = preg_replace('(^https?://)', '', URL_SHOP);
    $this->deviceIdentifier = 'Webserver';
    $this->exsID            = empty($infoxml->ExsID) ? null : (string) $infoxml->ExsID;

    $this->licenceCenterURL = preg_match('/.*\.dev\.fu[\/a-zA-Z\-_\.]*/', $this->hosting)
      ? 'https://cin-lizenzcenter.dev.fu/endpoint'
      : 'https://licence.cin-server.de/endpoint';
    $this->licenceFileName = \dirname(__DIR__, 1).'/'.$this->pluginid.'.lic';

    $this->loadLicenceFromDisk();
    if ($this->licence->mustnotrevalidatebefore < date('Y-m-d H:i:s')) {
      $this->callApi();
      $this->loadLicenceFromDisk();
    }
  }

  public function canRun(): bool
  {
    if (isset($this->exsID)) {
      return true;
    }

    $now          = strtotime(date('Y-m-d H:i:s'));
    $majorVersion = (int) explode('.', $this->version)[0];

    return $this->verifySignature()
      && strtotime($this->licence->notvalidbefore) <= $now
      && strtotime($this->licence->notvalidafter) >= $now
      && $majorVersion <= (int) $this->licence->validformajorversion
      && $this->licence->licensee === $this->hosting
      && $this->licence->productidentifier === $this->pluginid;
  }

  public function hasFeature($featureName): bool
  {
    return ('demo' === $this->licence->state && \in_array($featureName, array_keys((array) $this->licence->features), true))
      || (isset($this->exsID) && $this->verifySignature() && $this->licence->features->{$featureName})
      || $this->licence->features->{$featureName};
  }

  public function reissueLicence($authtoken): bool
  {
    $signedString = $this->hosting.$this->pluginid;
    $signature    = base64_decode($authtoken, true);

    if ($this->verifySignString($signedString, $signature)) {
      if ($this->unlinkLicenceFile()) {
        $this->loadLicenceFromDisk();

        return true;
      }

      return false;
    }

    return false;
  }

  public function getContent(): string
  {
    return json_encode($this->licence, \JSON_PRETTY_PRINT | \JSON_NUMERIC_CHECK | \JSON_UNESCAPED_SLASHES | \JSON_UNESCAPED_UNICODE);
  }

  private function unlinkLicenceFile(): bool
  {
    return unlink($this->licenceFileName);
  }

  private function verifySignature(): bool
  {
    $signee = clone $this->licence;
    unset($signee->signature);
    $signedString = json_encode($signee);
    $signature    = base64_decode($this->licence->signature, true);

    return $this->verifySignString($signedString, $signature);
  }

  private function verifySignString($signedString, $signature): bool
  {
    $pubkeyid = openssl_pkey_get_public($this->publicKey);
    $result   = openssl_verify($signedString, $signature, $pubkeyid, \OPENSSL_ALGO_SHA256);
    $pubkeyid = null;

    return 1 === $result;
  }

  private function loadLicenceFromDisk(): void
  {
    if ( ! file_exists($this->licenceFileName)) {
      $this->callApi();
    }
    $this->licence = json_decode(file_get_contents($this->licenceFileName), false);
  }

  private function callApi(): void
  {
    $payload = [
      'action' => 'requestLicence',
      'data'   => [
        'licensee'          => $this->hosting,
        'deviceidentifier'  => $this->deviceIdentifier,
        'productidentifier' => $this->pluginid,
        'productversion'    => $this->version,
      ],
    ];

    try {
      $cs = curl_init($this->licenceCenterURL);
      curl_setopt($cs, \CURLOPT_RETURNTRANSFER, true);
      curl_setopt($cs, \CURLOPT_CUSTOMREQUEST, 'POST');
      curl_setopt($cs, \CURLOPT_POSTFIELDS, json_encode($payload));
      curl_setopt($cs, \CURLOPT_HTTPHEADER, ['Content-Type:application/json']);
      $result = curl_exec($cs);
      curl_close($cs);
      if (false === $result) {
        throw new Exception('Error Processing Licence Request', 1);
      }
      $response = json_decode($result, false);
      if (false === $response || null === $response) {
        throw new Exception('Error Decoding Licence Response', 2);
      }
      file_put_contents($this->licenceFileName, json_encode($response->data->licence, \JSON_PRETTY_PRINT));
    } catch (Throwable $th) {
      if ( ! is_dir(\dirname(__DIR__, 1).'/log')) {
        mkdir(\dirname(__DIR__, 1).'/log');
      }

      file_put_contents(\dirname(__DIR__, 1).'/log/'.date('Y-m-d His').'.log', json_encode([
        'code'    => $th->getCode(),
        'message' => $th->getMessage(),
        'file'    => $th->getFile(),
        'line'    => $th->getLine(),
        'url'     => $this->licenceCenterURL,
        'payload' => $payload,
        'result'  => $result,
        'trace'   => $th->getTrace()
      ], \JSON_PRETTY_PRINT | \JSON_UNESCAPED_SLASHES));
      if ( ! file_exists($this->licenceFileName)) {
        $dummy                          = new stdClass();
        $dummy->productidentifier       = $this->pluginid;
        $dummy->notvalidbefore          = date('Y-m-d H:i:s');
        $dummy->notvalidafter           = date('Y-m-d H:i:s');
        $dummy->mustnotrevalidatebefore = date('Y-m-d H:i:s', strtotime('+1 hour', strtotime(date('c'))));
        $dummy->initialregistration     = null;
        $dummy->signature               = base64_encode('dummy');
        $dummy->state                   = 'dummy';
      } else {
        $dummy                          = json_decode(file_get_contents($this->licenceFileName));
        $dummy->notvalidbefore          = date('Y-m-d H:i:s');
        $dummy->notvalidafter           = date('Y-m-d H:i:s');
        $dummy->mustnotrevalidatebefore = date('Y-m-d H:i:s', strtotime('+1 hour', strtotime(date('c'))));
        $dummy->initialregistration     = null;
        $dummy->productidentifier       = $this->pluginid;
      }
      file_put_contents($this->licenceFileName, json_encode($dummy, \JSON_PRETTY_PRINT));
    }
  }
}
