<?php

declare(strict_types=1);

namespace Cin\Events5;

use JTL\Helpers\Text;
use JTL\Plugin\Helper as PluginHelper;
use JTL\Plugin\PluginInterface;
use JTL\Shop;
use PDO;
use SimpleXMLElement;
use stdClass;

final class Plugin
{
  private string $pluginid;

  private static PDO $pdo;

  private PluginInterface $oPlugin;

  public function __construct(?string $pluginid = null)
  {
    $infoxml = null === $pluginid
      ? (new SimpleXMLElement(file_get_contents(\dirname(__DIR__, 1).'/info.xml')))
      : (new SimpleXMLElement(file_get_contents(\dirname(__DIR__, 2)."/{$pluginid}/info.xml")));
    $this->pluginid = (string) $infoxml->PluginID[0];
  }

  public function cfg(): array
  {
    $configPath        = $this->getJTlPlugin()->getPaths()->getBasePath().'/common/settings/';
    $defaultConfigFile = $configPath.'default.php';
    $customConfigFile  = $configPath.'custom.php';

    $cfg = [];

    if (file_exists($defaultConfigFile)) {
      require $defaultConfigFile;
      foreach ($defaultConfig as $name => $value) {
        $cfg[$name] = $value;
      }
    }

    foreach ($this->getJTlPlugin()->getConfig()->getAssoc() as $name => $option) {
      $cfg[$name] = $option->value;
    }

    if (isset($defaultConfig) && file_exists($customConfigFile)) {
      require $customConfigFile;
      foreach ($customConfig as $name => $value) {
        if (\in_array($name, array_keys($cfg), true)) {
          $cfg[$name] = $value;
        }
      }
    }

    return $cfg;
  }

  public function getJTlPlugin(): PluginInterface
  {
    if ( ! class_exists('JTL\Plugin\Helper')) {
      require_once \dirname(__DIR__, 3).'/includes/globalinclude.php';
    }
    if (empty($this->oPlugin)) {
      $id            = PluginHelper::getIDByPluginID($this->pluginid);
      $this->oPlugin = PluginHelper::getLoaderByPluginID($id)->init($id);
    }

    return $this->oPlugin;
  }

  public function getPDO(): PDO
  {
    if (empty(self::$pdo)) {
      if ( ! \defined('DB_NAME')) {
        require \dirname(__DIR__, 3).'/includes/config.JTL-Shop.ini.php';
      }
      self::$pdo = new PDO(
        'mysql:dbname='.DB_NAME.';host='.DB_HOST.';charset=utf8mb4',
        DB_USER,
        DB_PASS,
        [
          PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_OBJ,
          PDO::ATTR_ERRMODE            => PDO::ERRMODE_EXCEPTION,
          PDO::ATTR_EMULATE_PREPARES   => false,
          PDO::ATTR_STRINGIFY_FETCHES  => false
        ]
      );
    }

    return self::$pdo;
  }

  public function getPluginID(): string
  {
    return $this->pluginid;
  }

  public function getSettingsTabID(string $identifier): string
  {
    $pdo  = $this->getPDO();
    $prep = $pdo->prepare('SELECT kPluginAdminMenu FROM tpluginadminmenu WHERE kPlugin = :kPlugin AND cName = :cName');

    $prep->execute([
      'kPlugin' => $this->getJTlPlugin()->getID(),
      'cName'   => $identifier
    ]);

    return (string) $prep->fetch(PDO::FETCH_COLUMN) ?: 0;
  }

  public function isForeignPluginInstalled(string $pluginid): bool
  {
    $stmt = $this->getPDO()->prepare('SELECT COUNT(*)
      FROM `tplugin`
      WHERE cPluginID = :pluginid AND nStatus = 2
    ');
    $stmt->execute([
      ':pluginid' => $pluginid
    ]);

    return 1 === $stmt->fetchColumn();
  }

  public function lang(): array
  {
    return $this->getJTlPlugin()->getLocalization()->getTranslations();
  }

  public function thirdPartyLibraries(): array
  {
    $partys = [];

    $composerFilename   = \dirname(__DIR__, 1).'/composer.lock';
    $partys['composer'] = file_exists($composerFilename) ? file_get_contents($composerFilename) : '{}';

    $thirdPartyFilename   = \dirname(__DIR__, 1).'/thirdparty.json';
    $partys['thirdparty'] = file_exists($thirdPartyFilename) ? file_get_contents($thirdPartyFilename) : '{}';

    return array_merge(json_decode($partys['composer'])->packages ?? [], json_decode($partys['thirdparty'])->packages ?? []);
  }

  public function translate(string $msgid): string
  {
    $getText = Shop::Container()->getGetText();
    $oldLang = $getText->getLanguage();
    $getText->setLanguage(Text::convertISO2ISO639(Shop::getLanguageCode()))->loadTranslations($this->getJTlPlugin()->getPaths()->getBasePath(), 'custom');

    $text = __($msgid);

    $getText->setLanguage($oldLang);

    return $text;
  }

  /**
   * Get a response object (stdClass) with 200 as default code.
   *
   * @param string   $message Is empty by default
   * @param stdClass $data    Is an empty stdClass object by default
   * @param string   $status  Has 'success' as default
   */
  public static function Response(?string $message = null, ?object $data = null, ?string $status = null): object
  {
    $response          = new stdClass();
    $response->data    = $data ?? new stdClass();
    $response->message = $message ?? '';
    $response->status  = $status ?? 'success';
    $response->code    = 200;

    return $response;
  }

  public function exitResponse(stdClass $response): void
  {
    if (isset($response->code)) {
      http_response_code($response->code);
    }

    if (isset($response->status) && 'error' === $response->status) {
      http_response_code(400);
    }

    header('Content-Type: application/json');
    exit(json_encode($response, 512));
  }
}
