<?php

declare(strict_types=1);

namespace Cin\Events5\service;

use DeepCopy\DeepCopy;
use JTL\Cart\CartItem;
use JTL\Shop;
use PDO;

final class EventCartService
{
  public static function buildComboKeysArray(array $oVariationKombi_arr): array
  {
    $comboValues = [];

    foreach ($oVariationKombi_arr as $combo) {
      $comboValues[] = $combo->kEigenschaft.':'.$combo->kEigenschaftWert;
    }

    return $comboValues;
  }

  /**
   * remove expired reservations.
   */
  public static function cleanReservations(): void
  {
    Shop::Container()
      ->getDB()
      ->getPDO()
      ->prepare('DELETE FROM cin_events_5_reserved WHERE expired_at < now()')
      ->execute();
  }

  public static function deleteReservedSeat(int $configID, string $position, string $identifier)
  {
    $pdo  = Shop::Container()->getDB()->getPDO();
    $prep = $pdo->prepare('DELETE FROM cin_events_5_reserved
      WHERE config_id = :cId AND position_id = :pId AND identifier = :identifier');

    return $prep->execute([
      ':cId'        => $configID,
      ':pId'        => $position,
      ':identifier' => $identifier
    ]);
  }

  public static function getEventData(CartItem $cartItem): ?string
  {
    foreach ($cartItem->WarenkorbPosEigenschaftArr as $k => $cartPosProperty) {
      if ('cin_event_data' === $cartPosProperty->cEigenschaftName['ger'] && isset($cartPosProperty->cEigenschaftWertName['ger'])) {
        return base64_decode($cartPosProperty->cEigenschaftWertName['ger'], true);
      }
    }

    return null;
  }

  public static function getEventDataObject(CartItem $cartItem): ?object
  {
    $d = self::getEventData($cartItem);

    return null === $d ? null : json_decode($d);
  }

  /**
   * Fetch all reserved tickets/places.
   *
   * @param bool $inCart FALSE only fetches those that are not yet in the shopping cart
   */
  public static function getReserved(int $configID, string $identifier, bool $inCart = false): array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT *
      FROM cin_events_5_reserved r
      WHERE r.config_id = :config_id
      AND r.identifier = :identifier
      AND r.in_cart = :incart';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':config_id'  => $configID,
      ':identifier' => $identifier,
      ':incart'     => ($inCart ? 1 : 0)
    ]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  /**
   * Check if property has name 'cin_event_data'.
   */
  public static function hasPropertyVarName($characteristicItem): bool
  {
    foreach ($characteristicItem->cEigenschaftName as $name) {
      if ('cin_event_data' === $name) {
        return true;
      }
    }

    return false;
  }

  /**
   * Checks if set a value for property.
   */
  public static function isSetVarNameValue($characteristicItem): bool
  {
    foreach ($characteristicItem->cEigenschaftWertName as $value) {
      if (isset($value) && mb_strlen($value) > 0) {
        return true;
      }
    }

    return false;
  }

  public static function removeItemFromReservationList(CartItem $cartItem, string $identifier): void
  {
    $j = self::getEventDataObject($cartItem);

    if ($j) {
      self::deleteReservedSeat((int) $j->cid, $j->place, $identifier);
    }
  }

  /**
   * Stores the new cart item with expiring time in cin_events_5_config.
   *
   * @return bool returns FALSE if no more ticket is available or on failure. TRUE if successfuly stored.
   */
  public static function reserveInCart(int $configID, int $count, string $identifier, string $position_id): bool
  {
    self::cleanReservations();

    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      c.contingent,
      c.contingent - IF(SUM(r.count) IS NULL, 0, SUM(r.count)) available
      FROM cin_events_5_config c
      LEFT JOIN cin_events_5_reserved r ON r.config_id = c.id
      WHERE c.id = :configID';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':configID' => $configID]);

    $rslt = $prep->fetch(PDO::FETCH_ASSOC);

    if ($rslt['available'] > 0 && ($rslt['available'] + $count) >= $rslt['contingent']) {
      $stmt = 'INSERT INTO cin_events_5_reserved (config_id, position_id, in_cart, `count`, identifier, expired_at)
        VALUES (:configID, :position_id, 1, :count, :identifier, :expired_at)';

      $prep = $db->getPDO()->prepare($stmt);

      return $prep->execute([
        ':configID'    => $configID,
        ':position_id' => $position_id,
        ':count'       => $count,
        ':identifier'  => $identifier,
        ':expired_at'  => date('Y-m-d H:i:s', strtotime('+30 minutes'))
      ]);
    }

    return false;
  }

  public static function reserveSeat(int $configID, string $position, string $identifier)//: bool
  {
    self::cleanReservations();

    $db = Shop::Container()->getDB();

    $stmt = 'SELECT r.position_id FROM cin_events_5_reserved r WHERE r.config_id = :configID';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':configID' => $configID]);

    $usedPlaces = $prep->fetchAll(PDO::FETCH_COLUMN);

    if ($usedPlaces && \in_array($position, $usedPlaces, true)) {
      return false;
    }

    $stmt = 'INSERT INTO cin_events_5_reserved (config_id, position_id, identifier, expired_at)
    VALUES (:configID, :position, :identifier, :expired_at)';

    $prep = $db->getPDO()->prepare($stmt);

    return $prep->execute([
      ':configID'   => $configID,
      ':position'   => $position,
      ':identifier' => $identifier,
      'expired_at'  => date('Y-m-d H:i:s', strtotime('+3 minutes')) // TODO
    ]);
  }

  /**
   * Split positions with multible count to single positions.
   */
  public static function splitMultiple(object &$position, array &$posArr): void
  {
    if ($position->nAnzahl > 1) {
      $detectedCount     = $position->nAnzahl;
      $position->nAnzahl = 1;

      for ($i = 1; $i < $detectedCount; ++$i) {
        $DC       = new DeepCopy();
        $posArr[] = $DC->skipUncloneable()->copy($position);
      }
    }
  }

  public static function stillAvailable(int $confID, string $position, string $identifier): bool
  {
    $pdo = Shop::Container()->getDB()->getPDO();

    $prep = $pdo->prepare('SELECT COUNT(*)
      FROM cin_events_5_reserved
      WHERE config_id = :confID AND position_id = :position AND identifier = :identifier');
    $prep->execute([
      ':confID'     => $confID,
      ':position'   => $position,
      ':identifier' => $identifier
    ]);

    if (1 === (int) ($prep->fetch(PDO::FETCH_COLUMN))) {
      return true;
    }

    $prep = $pdo->prepare('SELECT COUNT(*)
      FROM cin_events_5_reserved
      WHERE config_id = :confID AND position_id = :position');
    $prep->execute([
      ':confID'   => $confID,
      ':position' => $position
    ]);

    return ! (bool) (int) ($prep->fetch(PDO::FETCH_COLUMN));
  }

  public static function updateReserved($configID, $posID, $column, $value)
  {
    $db = Shop::Container()->getDB();

    $stmt = "UPDATE cin_events_5_reserved SET {$column} = :value WHERE config_id = :cId AND position_id = :pId";

    $prep = $db->getPDO()->prepare($stmt);

    return $prep->execute([
      ':cId'   => $configID,
      ':pId'   => $posID,
      ':value' => $value
    ]);
  }
}
