<?php

declare(strict_types=1);

namespace Cin\Events5\service;

use JTL\Shop;
use PDO;
use stdClass;

final class EventConfigService
{
  /**
   * Add an date for specific event.
   * In first check if date for event exists. Otherwise create the event and add new date/config.
   *
   * @param int $kVaterArtikel alias event
   */
  public static function addDate(int $kVaterArtikel, array $date, string &$errorMsg = ''): bool
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      c.id
      FROM cin_events_5_config c
      WHERE c.events_id = :kVaterArtikel
      AND c.`date` = :date';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':kVaterArtikel' => $kVaterArtikel, ':date' => $date['addDate']]);
    $ID = $prep->fetch(PDO::FETCH_COLUMN);

    if ($ID) {
      $errorMsg = __('dateExists');

      return false;
    }

    EventService::ensureEvent($kVaterArtikel);

    $stmt = 'INSERT INTO cin_events_5_config
      (events_id, `date`)
      VALUES (:kVaterArtikel, :date)';

    $prep = $db->getPDO()->prepare($stmt);

    return $prep->execute([
      ':kVaterArtikel' => $kVaterArtikel,
      ':date'          => $date['addDate']
    ]);
  }

  /**
   * Add begin and end times to specific event.
   *
   * @return bool true if statement successfully executed
   */
  public static function addTime(int $kVaterArtikel, array $data, string &$errorMsg = ''): bool
  {
    EventService::ensureEvent($kVaterArtikel);

    if (null === $data['addTimeBegin']) {
      return false;
    }

    $stmt = '
      INSERT INTO cin_events_5_config
      (events_id, `date`, `begin`, `end`)
      VALUES (:eID, :date, :begin, :end)';

    $prep = Shop::Container()->getDB()->getPDO()->prepare($stmt);

    return $prep->execute([
      ':eID'   => $kVaterArtikel,
      ':date'  => $data['forDate'],
      ':begin' => $data['addTimeBegin'],
      ':end'   => $data['addTimeEnd']
    ]);
  }

  /**
   * Makes a copy of one config line with new begin and end time.
   */
  public static function clone(int $configID, string $begin, ?string $end): bool
  {
    $pdo  = Shop::Container()->getDB()->getPDO();
    $stmt = 'INSERT INTO cin_events_5_config
      (events_id, `date`, `begin`, `end`, contingent, matrix_id)
      SELECT c.events_id, c.date, :begin, :end, c.contingent, c.matrix_id
      FROM cin_events_5_config c
      WHERE c.id = :cID';

    $prep = $pdo->prepare($stmt);
    $prep->execute([
      ':cID'   => $configID,
      ':begin' => $begin,
      ':end'   => $end
    ]);

    $last = $pdo->lastInsertId();

    $stmt = "INSERT INTO cin_events_5_matrix_var (config_id, svg_var, article_var)
      SELECT {$last}, cmv.svg_var, cmv.article_var
      FROM cin_events_5_matrix_var cmv
      WHERE cmv.config_id = :cID";

    $prep = $pdo->prepare($stmt);

    return $prep->execute([
      ':cID' => $configID
    ]);
  }

  /**
   * Make copies of all config lines where a specific date and event id is set.
   */
  public static function cloneDate(string $date, int $eventID, string $fromDate): bool
  {
    $pdo = EventService::Plugin()->getPDO();

    $prep = $pdo->prepare('SELECT id
      FROM cin_events_5_config c
      WHERE c.events_id = :eID
      AND c.date = :date');

    $prep->execute([
      ':eID'  => $eventID,
      ':date' => $fromDate
    ]);

    $cID_arr = $prep->fetchAll(PDO::FETCH_COLUMN);

    $success = true;

    foreach ($cID_arr as $cID) {
      $stmt = 'INSERT INTO cin_events_5_config
        (events_id, `date`, `begin`, `end`, contingent, matrix_id)
        SELECT c.events_id, :date, c.begin, c.end, c.contingent, c.matrix_id
        FROM cin_events_5_config c
        WHERE c.id = :cID';

      $prep = $pdo->prepare($stmt);
      $prep->execute([
        ':cID'  => $cID,
        ':date' => $date
      ]);

      $last = $pdo->lastInsertId();

      $stmt = "INSERT INTO cin_events_5_matrix_var (config_id, svg_var, article_var)
        SELECT {$last}, cmv.svg_var, cmv.article_var
        FROM cin_events_5_matrix_var cmv
        WHERE cmv.config_id = :cID";

      $prep    = $pdo->prepare($stmt);
      $success = $prep->execute([
        ':cID' => $cID
      ]) && $success;
    }

    return $success;
  }

  public static function delete(int $configID): stdClass
  {
    $pdo  = EventService::Plugin()->getPDO();
    $prep = $pdo->prepare('SELECT count(*) FROM cin_events_5_reserved WHERE config_id = :cID');

    $prep->execute([':cID' => $configID]);

    if ((int) $prep->fetchColumn() > 0) {
      return EventService::Plugin()->Response(__('hasReservations'), null, 'error');
    }

    $prep = $pdo->prepare('DELETE FROM cin_events_5_config WHERE id = :cID');

    return EventService::Plugin()->Response('', null, $prep->execute([':cID' => $configID]) ? 'success' : 'error');
  }

  public static function deleteDate(string $date, int $eventID): stdClass
  {
    $pdo = EventService::Plugin()->getPDO();

    $prep = $pdo->prepare('SELECT count(*)
      FROM cin_events_5_config c
      LEFT JOIN cin_events_5_reserved r ON r.config_id = c.id
      WHERE c.events_id = :eID
      AND c.date = :date
      AND r.identifier IS NOT NULL');

    $prep->execute([
      ':eID'  => $eventID,
      ':date' => $date
    ]);

    if ((int) $prep->fetchColumn() > 0) {
      return EventService::Plugin()->Response(__('hasReservations'), null, 'error');
    }

    $prep = $pdo->prepare('DELETE FROM cin_events_5_config
      WHERE events_id = :eID
      AND `date` = :date');

    return EventService::Plugin()->Response('', null, $prep->execute([
      ':eID'  => $eventID,
      ':date' => $date
    ]) ? 'success' : 'error');
  }

  public static function eventAvailableCount($configID)
  {
    EventCartService::cleanReservations();

    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      c.contingent - (
          IFNULL(
              (
                  SELECT SUM(sold.count)
                  FROM cin_events_5_config cs
                  LEFT JOIN cin_events_5_reserved sold ON sold.config_id = cs.id AND sold.sold = 1
                  WHERE cs.id = :id
              ),
              0
          )
          + IFNULL(
              (
                  SELECT SUM(r.count)
                  FROM cin_events_5_config cr
                  LEFT JOIN cin_events_5_reserved r ON r.config_id = cr.id AND r.sold = 0
                  WHERE cr.id = :id
              ),
              0
          )
      ) result
      FROM cin_events_5_config c
      WHERE c.id = :id';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':id' => $configID
    ]);

    return $prep->fetch(PDO::FETCH_COLUMN);
  }

  public static function getConfig($configID): array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      c.*,
      c.events_id kVaterArtikel
      FROM cin_events_5_config c
      #LEFT JOIN cin_events_5 e ON e.kVaterArtikel = c.events_id
      WHERE c.id = :id';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':id' => $configID
    ]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function getEventConfig($kVaterArtikel, $date, $begin, $end)
  {
    $db = Shop::Container()->getDB();

    $stmt = "SELECT
      c.id,
      c.`date`,
      c.`begin`,
      c.`end`,
      c.contingent,
      c.matrix_id,
      m.identifier,
      m.filename
      FROM cin_events_5_config c
      LEFT JOIN cin_events_5_matrix m ON m.id = c.matrix_id
      WHERE (c.`begin` = :begin OR c.`begin` IS NULL) AND (c.`end` = :end OR c.`end` = '00:00:00' OR c.`end` IS NULL)
      AND c.`date` = :date
      AND c.events_id = :kVaterArtikel";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':kVaterArtikel' => $kVaterArtikel,
      ':date'          => $date,
      ':begin'         => $begin,
      ':end'           => $end
    ]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function getEventDays($kVaterArtikel, bool $forceAll = true): array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT DISTINCT
      c.`date`
      FROM cin_events_5_config c
      WHERE c.events_id = :kVaterArtikel
      AND `date` > :date
      ORDER BY c.`date`';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([
      ':kVaterArtikel' => $kVaterArtikel,
      ':date'          => ($forceAll ? '2001-01-01' : date('Y-m-d'))
    ]);

    return $prep->fetchAll(PDO::FETCH_COLUMN);
  }

  public static function getEventTimes($kVaterArtikel, $date): array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      c.`begin`,
      c.`end`,
      c.id config
      FROM cin_events_5_config c
      WHERE c.date = :date
      AND c.events_id = :kVaterArtikel
      ORDER BY c.begin';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':date' => $date, ':kVaterArtikel' => $kVaterArtikel]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function getMonthlyData($date): array
  {
    $db = Shop::Container()->getDB();

    $stmt = "SELECT
      e.kVaterArtikel kVaterArtikel,
      e.image eventImage,
      #ced.kArtikel kVarArtikel,
      c.`date` 'date',
      c.`begin` 'begin',
      c.`end` 'end',
      c.id config,
      c.contingent,
      #c.reserved,
      #c.sold,
      c.matrix_id
      FROM cin_events_5 e
      LEFT JOIN cin_events_5_config c ON c.events_id = e.kVaterArtikel #AND (c.times_id = cet.id or c.times_id is null)
      WHERE c.`date` >= :date
      AND c.`date` < :date + INTERVAL 1 month
      ORDER BY c.`date`, c.`begin`";

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':date' => $date]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function hasEventAppointmentIDReservations(int $configID): bool
  {
    $pdo = EventService::Plugin()->getPDO();

    $prep = $pdo->prepare('SELECT count(*) FROM cin_events_5_reserved WHERE config_id = :cID');
    $prep->execute([':cID' => $configID]);

    return $prep->fetchColumn() > 0;
  }

  public static function hasEventAppointmentDateReservations(int $eventID, string $date): bool
  {
    $pdo = EventService::Plugin()->getPDO();

    $prep = $pdo->prepare('SELECT count(*)
      FROM cin_events_5_config c
      LEFT JOIN cin_events_5_reserved r ON r.config_id = c.id
      WHERE c.events_id = :eID
      AND c.date = :date
      AND r.identifier IS NOT NULL');

    $prep->execute([
      ':eID'  => $eventID,
      ':date' => $date
    ]);

    return $prep->fetchColumn() > 0;
  }

  public static function updateConfig(int $configID, $eventID, $date, $column, $value): stdClass
  {
    $pdo = EventService::Plugin()->getPDO();

    if (null === $configID) {
      $stmt = "INSERT INTO cin_events_5_config
      (`days_id`, `{$column}`)
      SELECT * FROM (
        SELECT d.id, :value
        FROM cin_events_5_days d
        WHERE d.date = :date
        AND d.event_id = :kVaterArtikel) as td";

      $execData = [
        ':kVaterArtikel' => $eventID,
        ':date'          => $date,
        ':value'         => $value
      ];
    } else {
      if (self::hasEventAppointmentIDReservations($configID)) {
        return EventService::Plugin()->Response(__('hasReservations'), null, 'error');
      }

      $stmt = "UPDATE cin_events_5_config c
      SET c.`{$column}` = :value
      WHERE c.id = :cID";

      $execData = [
        ':cID'   => $configID,
        ':value' => $value
      ];
    }

    $prep = $pdo->prepare($stmt);

    return EventService::Plugin()->Response('', null, $prep->execute($execData) ? 'success' : 'error');
  }

  /**
   * @param string $old     The date that should be change
   * @param string $new     The new date
   * @param int    $eventID kArtikel of (father)article
   *
   * @return ServiceResponse Response object with Status & Message string and a data object
   */
  public static function updateDate(string $old, string $new, int $eventID): stdClass
  {
    $pdo = EventService::Plugin()->getPDO();

    if (self::hasEventAppointmentDateReservations($eventID, $old)) {
      return EventService::Plugin()->Response(__('hasReservations'), null, 'error');
    }

    $stmt = 'UPDATE cin_events_5_config c
    SET c.`date` = :new
    WHERE c.`date` = :old
    AND c.events_id = :eID';

    $execData = [
      ':eID' => $eventID,
      ':old' => $old,
      ':new' => $new
    ];

    $prep = $pdo->prepare($stmt);

    return EventService::Plugin()->Response('', null, $prep->execute($execData) ? 'success' : 'error');
  }
}
