<?php

declare(strict_types=1);

namespace Cin\Events5\service;

use JTL\Shop;
use PDO;

final class EventMatrixService
{
  public const EVENT_NO_IMAGE = '/gfx/keinBild.gif';

  public const EVENT_USER_IMAGE_PATH = '/mediafiles/cin_events_5/';

  public static function getEventMatrixData($configID)
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      r.sold,
      r.position_id,
      r.count,
      r.identifier user,
      r.in_cart,
      m.identifier,
      m.filename
      FROM cin_events_5_config c
      LEFT JOIN cin_events_5_matrix m ON m.id = c.matrix_id
      LEFT JOIN cin_events_5_reserved r ON c.id = r.config_id
      WHERE c.id = :configID';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':configID' => $configID]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function getMatrixByIdentifier(string $identifier): array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT * FROM cin_events_5_matrix WHERE identifier = :identifier';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':identifier' => $identifier]);

    return $prep->fetchAll(PDO::FETCH_ASSOC);
  }

  public static function getMatrixList(): array
  {
    $stmt = 'SELECT * FROM cin_events_5_matrix';

    $prep = EventService::Plugin()->getPDO()->prepare($stmt);
    $prep->execute();

    return $prep->fetchAll(PDO::FETCH_ASSOC | PDO::FETCH_UNIQUE);
  }

  public static function getMatrixVar($configID, ?string $svg_var = null): ?array
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      mv.svg_var,
      mv.article_var
      FROM cin_events_5_matrix_var mv
      WHERE mv.config_id = :configID';

    $stmt_vars = [':configID' => $configID];

    if (null !== $svg_var) {
      $stmt .= ' AND mv.svg_var = :svg_var';
      $stmt_vars[':svg_var'] = $svg_var;
    }

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute($stmt_vars);

    return $prep->fetchAll(PDO::FETCH_ASSOC) ?: null;
  }

  public static function getMatrixPlaceVar($matrixID, $place): ?string
  {
    $db = Shop::Container()->getDB();

    $stmt = 'SELECT
      mvl.place_var
      FROM cin_events_5_matrix_var_link mvl
      WHERE mvl.matrix_id = :matrix_id
      AND mvl.place = :place';

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute([':matrix_id' => $matrixID, ':place' => $place]);

    return $prep->fetch(PDO::FETCH_COLUMN) ?: null;
  }

  public static function hasLinkedReservations(string $matrixFileName): bool
  {
    $stmt = 'SELECT count(*) c
    FROM cin_events_5_matrix m
    LEFT JOIN cin_events_5_config c ON c.matrix_id = m.id
    LEFT JOIN cin_events_5_reserved r ON r.config_id = c.id
    WHERE m.filename = :matrixFileName
    AND r.identifier IS NOT NULL';

    $prep = EventService::Plugin()->getPDO()->prepare($stmt);
    $prep->execute([
      ':matrixFileName' => $matrixFileName
    ]);

    return $prep->fetchColumn() > 0;
  }

  public static function isLinkedToConfig(string $matrixFileName): bool
  {
    $stmt = 'SELECT count(*) c
    FROM cin_events_5_matrix m
    LEFT JOIN cin_events_5_config c ON c.matrix_id = m.id
    WHERE m.filename = :matrixFileName
    AND c.matrix_id IS NOT NULL';

    $prep = EventService::Plugin()->getPDO()->prepare($stmt);
    $prep->execute([
      ':matrixFileName' => $matrixFileName
    ]);

    return $prep->fetchColumn() > 0;
  }

  public static function linkSvg($svgID, $data): void
  {
    $db = Shop::Container()->getDB();

    $stmt      = 'INSERT INTO cin_events_5_matrix_var_link (matrix_id, place, place_var) VALUES ';
    $stmt_vars = [':matrix_id' => $svgID];

    for ($i = 0; $i < \count($data); ++$i) {
      $stmt .= ($i > 0 ? ',' : '')."(:matrix_id, :place_{$i}, :place_var_{$i})";
      $stmt_vars[":place_{$i}"]     = $data[$i]['place'];
      $stmt_vars[":place_var_{$i}"] = $data[$i]['place-var'];
    }

    $prep = $db->getPDO()->prepare($stmt);
    $prep->execute($stmt_vars);
  }

  public static function storeMatrix($identifier, $filename): bool
  {
    $db = Shop::Container()->getDB();

    $stmt = 'INSERT INTO cin_events_5_matrix (identifier, filename)
      VALUES (:identifier, :filename);';

    $prep = $db->getPDO()->prepare($stmt);

    return $prep->execute([':identifier' => $identifier, ':filename' => $filename]);
  }

  public static function updateMatrixName(?string $identifier, string $filename): bool
  {
    $dbo = Shop::Container()->getDB()->getPDO();

    $execData = [
      ':identifier' => $identifier ?: $filename,
      ':filename'   => $filename.'%'
    ];

    $prep = $dbo->prepare('UPDATE cin_events_5_matrix SET identifier = :identifier WHERE filename LIKE :filename');

    return $prep->execute($execData);
  }

  public static function updateMatrixVar($configID, $column, $value): bool
  {
    $db = Shop::Container()->getDB();

    if (null === $value) {
      $stmt     = 'DELETE FROM cin_events_5_matrix_var WHERE config_id = :configID AND svg_var = :svg';
      $execData = [
        ':configID' => $configID,
        ':svg'      => $column
      ];
    } else {
      $stmt = 'INSERT INTO cin_events_5_matrix_var
      (`config_id`, `svg_var`, `article_var`)
      VALUES (:configID, :svg, :article)
      ON DUPLICATE KEY UPDATE article_var = :article';

      $execData = [
        ':configID' => $configID,
        ':svg'      => $column,
        ':article'  => $value
      ];
    }

    $prep = $db->getPDO()->prepare($stmt);

    return $prep->execute($execData);
  }
}
